/*
**      DBExpression.h
**$	F  Database Kit, Release 3.0
**      Copyright (c) 1992, NeXT Computer, Inc.  All rights reserved. 
*/

#import <objc/Object.h>
#import <dbkit/protocols.h>

/*
** The DBExpression represents a property of some DBDatabase; this property
**  is expressed in the underlying query language.
**
** initForEntity:fromDescription: results in a property that represents
**  "derived" data -- it can contain formulae, functions, or anything that
**  the underlying query language can support.  See DBQualifier.h for a
**  description of the format string used by both.
**
** initForEntity:fromName:usingType: results in a "simple" property that
**  represents a property from the database -- but can have a type provided.
**  So, for instance, if the underlying database supports a huge number
**  format, which must be represented as a string in objective-C in order
**  not to lose precision, this call could be used to generate a property
**  of "*" type.  The name argument must represent a named property of the
**  owner argument.
*/
@interface DBExpression : Object <DBProperties, DBExpressionValues>
{
@private
  id _entity;
  id _property;
}

- initForEntity:(id<DBEntities>)anEntity
    fromDescription:(const unsigned char*)aFormatString, ... ;
- initForEntity:(id<DBEntities>)anEntity
    fromName:(const char*)aName usingType:(const char*)aType;
- copyFromZone:(NXZone*)z;
- free;

- setEntity:(id<DBEntities>)anEntity 
    andDescription:(const unsigned char*)aFormatString, ...;

- read:(NXTypedStream*)ts;
- write:(NXTypedStream*)ts;

@end
